package main

import (
	"bufio"
	"bytes"
	"compress/gzip"
	"database/sql"
	"encoding/json"
	"fmt"

	//"html"
	"html/template"
	"io"
	"io/fs"
	"log"
	"math"
	"math/rand"
	"net/http"
	"net/url"
	"os"
	"path/filepath"
	"regexp"
	"sort"
	"strconv"
	"strings"
	"time"

	"golang.org/x/net/html"
	"golang.org/x/net/html/charset"
	"golang.org/x/text/encoding/simplifiedchinese"
	"golang.org/x/text/transform"
)

// web
func (wd *WebData) R0() map[string]map[string]string {
	wd.lock.RLock()
	defer wd.lock.RUnlock()
	return wd.data
}
func (wd *WebData) R(key string) map[string]string {
	wd.lock.RLock()
	value := wd.data[key]
	wd.lock.RUnlock()
	return value
}
func (wd *WebData) W(key string, value map[string]string) {
	wd.lock.Lock()
	if _, ok := wd.data[key]; !ok {
		wd.data[key] = make(map[string]string)
	}
	wd.data[key] = value
	wd.lock.Unlock()
}
func (wd *WebData) D(key string) {
	wd.lock.Lock()
	delete(wd.data, key)
	wd.lock.Unlock()
}

// channel
func (cd *ChannelData) R0() map[string]map[string]string {
	cd.lock.RLock()
	defer cd.lock.RUnlock()
	return cd.data
}
func (cd *ChannelData) R(key string) map[string]string {
	cd.lock.RLock()
	value := cd.data[key]
	cd.lock.RUnlock()
	return value
}
func (cd *ChannelData) W(key string, value map[string]string) {
	cd.lock.Lock()
	if _, ok := cd.data[key]; !ok {
		cd.data[key] = make(map[string]string)
	}
	cd.data[key] = value
	cd.lock.Unlock()
}
func (cd *ChannelData) R2(key, key2 string) map[string]string {
	cd.lock.RLock()
	value := cd.data[key][key2]
	cd.lock.RUnlock()
	var csj map[string]string
	json.Unmarshal([]byte(value), &csj)
	return csj
}
func (cd *ChannelData) W2(key, key2 string, value map[string]string) {
	v, _ := json.Marshal(value)
	cd.lock.Lock()
	if _, ok := cd.data[key]; !ok {
		cd.data[key] = make(map[string]string)
	}
	cd.data[key][key2] = string(v)
	cd.lock.Unlock()
}
func (cd *ChannelData) W3(key, key2, value string) { // 数据库直接存
	cd.lock.Lock()
	if _, ok := cd.data[key]; !ok {
		cd.data[key] = make(map[string]string)
	}
	cd.data[key][key2] = value
	cd.lock.Unlock()
}
func (cd *ChannelData) D(key string) {
	cd.lock.Lock()
	delete(cd.data, key)
	cd.lock.Unlock()
}
func (cd *ChannelData) D2(key, key2 string) {
	cd.lock.Lock()
	delete(cd.data[key], key2)
	cd.lock.Unlock()
}

// article
func (ad *ArticleData) R0() map[string]map[string]string {
	ad.lock.RLock()
	defer ad.lock.RUnlock()
	return ad.data
}
func (ad *ArticleData) R(key string) map[string]string {
	ad.lock.RLock()
	value := ad.data[key]
	ad.lock.RUnlock()
	return value
}
func (ad *ArticleData) W(key string, value map[string]string) {
	ad.lock.Lock()
	if _, ok := ad.data[key]; !ok {
		ad.data[key] = make(map[string]string)
	}
	ad.data[key] = value
	ad.lock.Unlock()
}
func (ad *ArticleData) R2(key, key2 string) map[string]string {
	ad.lock.RLock()
	value := ad.data[key][key2]
	ad.lock.RUnlock()
	var asj map[string]string
	json.Unmarshal([]byte(value), &asj)
	return asj
}
func (ad *ArticleData) W2(key, key2 string, value map[string]string) {
	v, _ := json.Marshal(value)
	ad.lock.Lock()
	if _, ok := ad.data[key]; !ok {
		ad.data[key] = make(map[string]string)
	}
	ad.data[key][key2] = string(v)
	ad.lock.Unlock()
}
func (ad *ArticleData) W3(key, key2, value string) { // 数据库直接存
	ad.lock.Lock()
	if _, ok := ad.data[key]; !ok {
		ad.data[key] = make(map[string]string)
	}
	ad.data[key][key2] = value
	ad.lock.Unlock()
}
func (cd *ArticleData) D(key string) {
	cd.lock.Lock()
	delete(cd.data, key)
	cd.lock.Unlock()
}
func (cd *ArticleData) D2(key, key2 string) {
	cd.lock.Lock()
	delete(cd.data[key], key2)
	cd.lock.Unlock()
}

// user
func (ud *UserData) R0() map[string]*Conn {
	ud.lock.RLock()
	defer ud.lock.RUnlock()
	return ud.data
}
func (ud *UserData) R(key string) (*Conn, bool) {
	ud.lock.RLock()
	value, ok := ud.data[key]
	ud.lock.RUnlock()
	return value, ok
}
func (ud *UserData) W(key string, value *Conn) {
	ud.lock.Lock()
	ud.data[key] = value
	ud.lock.Unlock()
}
func (ud *UserData) D(key string) {
	ud.lock.Lock()
	delete(ud.data, key)
	ud.lock.Unlock()
}

// channellb
func (cld *ChannellbData) R0() map[string][]string {
	cld.lock.RLock()
	defer cld.lock.RUnlock()
	return cld.data
}
func (cld *ChannellbData) R(key string) []string {
	cld.lock.RLock()
	value := cld.data[key]
	cld.lock.RUnlock()
	return value
}
func (cld *ChannellbData) W(key string, value []string) {
	cld.lock.Lock()
	cld.data[key] = value
	cld.lock.Unlock()
}
func (cld *ChannellbData) D(key string) {
	cld.lock.Lock()
	delete(cld.data, key)
	cld.lock.Unlock()
}

// webconfig
func (wcd *WebconfigData) R0() map[string]map[string]string {
	wcd.lock.RLock()
	defer wcd.lock.RUnlock()
	return wcd.data
}
func (wcd *WebconfigData) R(key string) (map[string]string, bool) {
	wcd.lock.RLock()
	value, ok := wcd.data[key]
	wcd.lock.RUnlock()
	return value, ok
}
func (wcd *WebconfigData) W(key string, value map[string]string) {
	wcd.lock.Lock()
	if _, ok := wcd.data[key]; !ok {
		wcd.data[key] = make(map[string]string)
	}
	wcd.data[key] = value
	wcd.lock.Unlock()
}
func (wcd *WebconfigData) D(key string) {
	wcd.lock.Lock()
	delete(wcd.data, key)
	wcd.lock.Unlock()
}
func (ipd *IpjiluData) R(key string) (int64, bool) {
	ipd.lock.RLock()
	value, ok := ipd.data[key]
	ipd.lock.RUnlock()
	return value, ok
}
func (ipd *IpjiluData) W(key string, value int64) {
	ipd.lock.Lock()
	ipd.data[key] = value
	ipd.lock.Unlock()
}

// weblb
func (wld *WeblbData) R() []Weblb {
	wld.lock.RLock()
	defer wld.lock.RUnlock()
	return wld.data
}
func (wld *WeblbData) W(value []Weblb) {
	wld.lock.Lock()
	wld.data = value
	wld.lock.Unlock()
}

// 初始化数据
func chushihua() {
	//初始化全局变量
	// 读取配置文件 如果失败初始化默认配置
	if !getconfig() {
		port = "12345"                                                       //监听端口
		dbhost = "127.0.0.1"                                                 // 数据库地址
		dbport = "3306"                                                      // 数据库端口
		dbuser = "root"                                                      // 数据库用户名
		dbpass = "123456"                                                    // 数据库密码
		dbname = "yibaocms"                                                  // 数据库名
		safeurl = "yibaocms"                                                 // 后台地址
		safeuser = "yibaocms"                                                // 后台用户名
		safepass = "yibaocms123"                                             // 后台密码
		dsn = dbuser + ":" + dbpass + "@tcp(" + dbhost + ":" + dbport + ")/" // dsn
	} //读取配置文件
	Db1, _ = sql.Open("mysql", dsn) //open不会检验用户名和密码
	dsn = dbuser + ":" + dbpass + "@tcp(" + dbhost + ":" + dbport + ")/" + dbname
	Db, _ = sql.Open("mysql", dsn)
	// 设置连接池参数
	Db.SetMaxIdleConns(5)                  // 最大空闲连接数
	Db.SetMaxOpenConns(20)                 // 最大打开连接数
	Db.SetConnMaxLifetime(time.Minute * 3) // 连接的最大生命周期
	//获取最后保存的数据路径
	var ok = true
	//最后一次保存文件路径 当数据库无法使用时，程序异常退出，将所有数据保存在一个临时文件中，并将这个文件路径记录在一个文件中
	backfilepath, err := os.ReadFile("./backup/backup.txt")
	if string(backfilepath) != "" && err == nil {
		allshuju, err := os.ReadFile(string(backfilepath))
		if err != nil {
			ok = false
		}
		//解压
		var buf bytes.Buffer
		buf.Write(allshuju)
		gr, err := gzip.NewReader(&buf)
		if err != nil {
			ok = false
		}
		defer gr.Close()
		allshujus, err := io.ReadAll(gr)
		if err != nil {
			ok = false
		}
		if ok {
			var backsj Backjson
			json.Unmarshal(allshujus, &backsj)
			web = &WebData{
				data: backsj.Web,
			}
			channel = &ChannelData{
				data: backsj.Channel,
			}
			article = &ArticleData{
				data: backsj.Article,
			}
			channellb = &ChannellbData{
				data: backsj.Channellb,
			}
			weblb = &WeblbData{
				data: backsj.Weblb,
			}
			webconfig = &WebconfigData{
				data: backsj.Webconfig,
			}
			os.Remove("./backup/backup.txt") //删除文件
			checkdb()                        //检查数据库
		}
	} else {
		ok = false
	}
	if !ok {
		//检查数据表
		if checkdb() {
			//初始化站点列表
			rows, err := Db.Query("SELECT * FROM web")
			if err != nil {
				panic(err.Error())
			}
			defer rows.Close()
			for rows.Next() {
				var id, webzd, channelzd, articlezd, flinks, zaxiang, post, lschannellb string
				if err := rows.Scan(&id, &webzd, &channelzd, &articlezd, &flinks, &zaxiang, &post, &lschannellb); err != nil {
					panic(err.Error())
				}
				//将webzd序列化
				var lswebzd []Ziduan
				json.Unmarshal([]byte(webzd), &lswebzd)
				lsweblb := weblb.R()
				//获取link
				for _, v := range lswebzd {
					if v.K == "link" {
						lsweblb = append(lsweblb, Weblb{I: id, U: v.V}) //添加到站点列表
						break
					}
				}
				weblb.W(lsweblb)
				lsweb := map[string]string{
					"webzd":     webzd,
					"channelzd": channelzd,
					"articlezd": articlezd,
					"flinks":    flinks,
					"zaxiang":   zaxiang,
					"post":      post,
					"channellb": lschannellb,
				}
				web.W(id, lsweb)
				lschannel := strings.Split(lschannellb, ",") //转切片
				if lschannel[0] != "" {
					channellb.W(id, lschannel)
				} else {
					channellb.W(id, []string{})
				}
				var sj2 []Ziduan
				json.Unmarshal([]byte(webzd), &sj2)
				var url, dir, muban string
				for _, v := range sj2 {
					if v.K == "link" {
						url = v.V
					}
					if v.K == "dir" {
						dir = v.V
					}
				}
				var sj3 map[string]string
				json.Unmarshal([]byte(zaxiang), &sj3)
				muban = sj3["muban"]
				_, host, _, _ := parseURL(url)
				if host != "" {
					lswebconfig := map[string]string{
						"id":    id,
						"dir":   dir,
						"muban": muban,
					}
					webconfig.W(host, lswebconfig)
				}
			}
			if err := rows.Err(); err != nil {
				panic(err.Error())
			}
			//初始化网站栏目
			rows, err = Db.Query("SELECT * FROM channel")
			if err != nil {
				panic(err.Error())
			}
			defer rows.Close()
			for rows.Next() {
				var id, webid, vid, topid, shuju string
				if err := rows.Scan(&id, &webid, &vid, &topid, &shuju); err != nil {
					panic(err.Error())
				}
				if webid != "0" && vid != "0" {
					channel.W3(webid, vid, shuju) //写入栏目数据
				}
			}
			//初始化文章
			rows, err = Db.Query("SELECT * FROM article")
			if err != nil {
				panic(err.Error())
			}
			defer rows.Close()
			for rows.Next() {
				var id, webid, aid, shuju string
				if err := rows.Scan(&id, &webid, &aid, &shuju); err != nil {
					panic(err.Error())
				}
				if webid != "0" && aid != "0" {
					article.W3(webid, aid, shuju) //写入文章数据
				}
			}
		} else {
			panic("数据库表连接创建失败！")
		}
	}
}

// 获取配置
func getconfig() bool {
	// 打开文件
	file, err := os.Open("config.txt")
	if err != nil {
		return false
	}
	defer file.Close()
	scanner := bufio.NewScanner(file) // 创建一个扫描器用于逐行读取文件
	// 逐行读取并处理
	for scanner.Scan() {
		line := scanner.Text()
		parts := strings.SplitN(line, "=", 2) // 分割每行，最多分为两部分
		if len(parts) != 2 {
			continue // 跳过格式不正确的行
		}
		key := parts[0]
		value := parts[1]
		switch key {
		case "port":
			port = value
		case "dbhost":
			dbhost = value
		case "dbport":
			dbport = value
		case "dbuser":
			dbuser = value
		case "dbpass":
			dbpass = value
		case "dbname":
			dbname = value
		case "safeurl":
			safeurl = value
		case "safeuser":
			safeuser = value
		case "safepass":
			safepass = value
		}
	}
	dsn = dbuser + ":" + dbpass + "@tcp(" + dbhost + ":" + dbport + ")/"
	return true
}

// 检查数据库
func checkdb() bool {
	Db1.Ping()
	Db.Ping()
	ok := true
	// 检查数据库是否存在
	rows, err := Db1.Query("SHOW DATABASES LIKE '" + dbname + "'")
	if err != nil {
		ok = false
		log.Println("检查主数据库出错:", err)
	} else {
		if !rows.Next() {
			// 数据库不存在，创建数据库
			log.Println("主数据库不存在...开始创建主数据库")
			_, err := Db1.Exec("CREATE DATABASE " + dbname)
			if err != nil {
				ok = false
				log.Println("创建主数据库出错:", err)
			} else {
				log.Println("主数据库创建成功")
			}

		}
		rows.Close()
	}
	// 检查web表是否存在
	rows, err = Db.Query("SHOW TABLES LIKE 'web'")
	if err != nil {
		ok = false
		log.Println("检查数据库表web出错:", err)
	} else {
		if !rows.Next() {
			// 表不存在，创建表
			log.Println("数据库web表不存在...开始创建web表")
			_, err := Db.Exec("CREATE TABLE web (id INT PRIMARY KEY AUTO_INCREMENT, webzd MEDIUMTEXT,channelzd MEDIUMTEXT,articlezd MEDIUMTEXT,flinks MEDIUMTEXT,zaxiang MEDIUMTEXT,post MEDIUMTEXT,channellb MEDIUMTEXT)")
			if err != nil {
				ok = false
				log.Println("创建web表出错:", err)
			} else {
				log.Println("创建web表成功")
			}

		}
		rows.Close()
	}
	// 检查channel表是否存在
	rows, err = Db.Query("SHOW TABLES LIKE 'channel'")
	if err != nil {
		ok = false
		log.Println("检查数据库表channel出错:", err)
	} else {
		if !rows.Next() {
			// 表不存在，创建表
			log.Println("数据库channel表不存在...开始创建channel表")
			_, err := Db.Exec("CREATE TABLE channel (id INT PRIMARY KEY AUTO_INCREMENT, webid INT,vid INT,topid INT,shuju MEDIUMTEXT,UNIQUE (webid, vid))")
			if err != nil {
				ok = false
				log.Println("创建channel表出错:", err)
			} else {
				log.Println("创建channel表成功")
			}
		}
		rows.Close()
	}
	// 检查article表是否存在
	rows, err = Db.Query("SHOW TABLES LIKE 'article'")
	if err != nil {
		ok = false
		log.Println("检查数据库表article出错:", err)
	} else {
		if !rows.Next() {
			// 表不存在，创建表
			log.Println("数据库article表不存在...开始创建article表")
			_, err := Db.Exec("CREATE TABLE article (id INT PRIMARY KEY AUTO_INCREMENT, webid INT,aid INT,shuju MEDIUMTEXT,UNIQUE (webid, aid))")
			if err != nil {
				ok = false
				log.Println("创建article表出错:", err)
			} else {
				log.Println("创建article表成功")
			}
		}
		rows.Close()
	}
	if ok {
		return true
	} else {
		return false
	}
}

// 删除非中文、字母和数字
func removeNonChineseAlphanumeric(s string) string {
	re := regexp.MustCompile(`[^\p{L}\p{N}]+`)
	return re.ReplaceAllStringFunc(html.EscapeString(s), func(r string) string {
		// 保留中文字符
		if len(r) == 1 && r >= "\u4e00" && r <= "\u9fff" {
			return r
		}
		// 移除非中文、字母和数字的字符
		return ""
	})
}

// 删除所有不是中文、字母和数字的字符
func removeInvalidChars(s string) string {
	re := regexp.MustCompile(`[^\p{Han}a-zA-Z0-9]`)      // 定义正则表达式，匹配所有非中文、字母和数字的字符
	return re.ReplaceAllString(html.EscapeString(s), "") // 使用正则表达式替换所有匹配的字符
}

// 删除所有不是字母和数字的字符
func removeNonAlphanumeric(s string) string {
	re := regexp.MustCompile(`[^a-zA-Z0-9]`)             // 定义正则表达式，匹配所有非字母和非数字的字符
	return re.ReplaceAllString(html.EscapeString(s), "") // 使用正则表达式替换所有匹配的字符
}

// 删除所有不是数字的字符串
func removeNonNumeric(s string) string {
	re := regexp.MustCompile(`[^0-9]`)                // 定义正则表达式，匹配所有非数字的字符
	s = re.ReplaceAllString(html.EscapeString(s), "") // 使用正则表达式替换所有匹配的字符
	if len(s) == 0 {
		return ""
	}
	if s[0] == '0' {
		return ""
	}
	return s
}

// 删除所有不是数字和_的字符串
func removeNonNumericAnd_(s string) string {
	re := regexp.MustCompile(`[^0-9_]`)               // 定义正则表达式，匹配所有非数字和_的字符
	s = re.ReplaceAllString(html.EscapeString(s), "") // 使用正则表达式替换所有匹配的字符
	if len(s) == 0 {
		return ""
	}
	if s[0] == '0' {
		return ""
	}
	return s
}

// 删除所有不是中文、字母和数字的字符串 用于搜索
func removeNonChinese(s string) string {
	s = strings.ReplaceAll(html.EscapeString(s), " ", ",")
	s = strings.ReplaceAll(s, "，", ",")
	s = strings.ReplaceAll(s, "-", ",")
	re := regexp.MustCompile(`[^a-zA-Z0-9,_\p{Han}]+`)
	return re.ReplaceAllString(s, "")
}

// 判断网址是否有效
func isUrl(s string) bool {
	u, err := url.ParseRequestURI(s)
	if err != nil {
		return false
	}
	if u.Scheme == "" || u.Host == "" {
		return false
	}
	return true
}

// 获取网址协议 host 路径
func parseURL(s string) (string, string, string, error) {
	u, err := url.Parse(s)
	if err != nil {
		return "", "", "", err
	}
	scheme := u.Scheme
	host := u.Host
	path := u.Path
	return scheme, host, path, nil
}

// copyFile 拷贝单个文件
func copyfile(src, dst string) (int64, error) {
	sourceFileStat, err := os.Stat(src)
	if err != nil {
		return 0, err
	}

	// 如果源文件不存在
	if !sourceFileStat.Mode().IsRegular() {
		return 0, fmt.Errorf("%s is not a regular file", src)
	}

	source, err := os.Open(src)
	if err != nil {
		return 0, err
	}
	defer source.Close()

	destination, err := os.Create(dst)
	if err != nil {
		return 0, err
	}
	defer destination.Close()
	nBytes, err := io.Copy(destination, source)
	return nBytes, err
}

// copyDir 拷贝整个目录
func copydir(src string, dst string) error {
	srcInfo, err := os.Stat(src)
	if err != nil {
		return err
	}
	err = os.MkdirAll(dst, srcInfo.Mode())
	if err != nil {
		return err
	}
	dirEntries, err := os.ReadDir(src)
	if err != nil {
		return err
	}
	for _, entry := range dirEntries {
		srcPath := filepath.Join(src, entry.Name())
		dstPath := filepath.Join(dst, entry.Name())
		if entry.IsDir() {
			err = copydir(srcPath, dstPath)
			if err != nil {
				return err
			}
		} else {
			_, err = copyfile(srcPath, dstPath)
			if err != nil {
				return err
			}
		}
	}
	return nil
}

// 删除指定目录
func deleteDirectory(dir string) error {
	err := filepath.Walk(dir, func(path string, info os.FileInfo, err error) error {
		if err != nil {
			return err
		}
		if info.IsDir() {
			return nil
		}
		return os.Remove(path)
	})
	if err != nil {
		return err
	}
	return os.RemoveAll(dir)
}

// 获取指定目录及其子目录下的所有文件
func listFiles(dir string) ([]string, error) {
	var files []string
	// 使用 filepath.WalkDir 遍历目录
	err := filepath.WalkDir(dir, func(path string, d fs.DirEntry, err error) error {
		if err != nil {
			return err
		}
		if !d.IsDir() {
			files = append([]string{path}, files...)
		}
		return nil
	})
	if err != nil {
		return nil, err
	}
	return files, nil
}

// 获取文件尺寸
func getFileSize(filePath string) (float64, error) {
	fileInfo, err := os.Stat(filePath)
	if err != nil {
		return 0, err
	}
	fileSize := fileInfo.Size()
	// 转换为KB
	fileSizeInKB := float64(fileSize) / 1024.0

	// 返回保留两位小数的结果
	return fileSizeInKB, nil
}

// 获取指定目录下的子目录
func getSubDirectories(dirPath string) ([]string, error) {
	var subDirs []string
	entries, err := os.ReadDir(dirPath)
	if err != nil {
		return nil, err
	}
	for _, entry := range entries {
		if entry.IsDir() {
			subDirs = append(subDirs, entry.Name())
		}
	}
	return subDirs, nil
}

// 判断文件是否存在
func fileExists(filePath string) bool {
	_, err := os.Stat(filePath)
	if err == nil {
		return true
	}
	if os.IsNotExist(err) {
		return false
	}
	return false
}

// 删除站点
func Deletesite(webid string) bool {
	Db.Ping() //测试连接
	lsweblb := weblb.R()
	//删除站点列表中的webid
	for i, v := range lsweblb {
		if v.I == webid {
			lsweblb = append(lsweblb[:i], lsweblb[i+1:]...)
			break
		}
	}
	weblb.W(lsweblb)
	lsweb := web.R(webid)
	var sj []Ziduan
	json.Unmarshal([]byte(lsweb["webzd"]), &sj)
	var url, dir string
	for _, v := range sj {
		if v.K == "link" {
			url = v.V
		}
		if v.K == "dir" {
			dir = v.V
		}
	}
	_, host, _, _ := parseURL(url)
	if host != "" {
		webconfig.D(host) //删除web配置
	}
	//删除缓存
	web.D(webid)
	channel.D(webid)
	article.D(webid)
	channellb.D(webid)
	//删除数据库
	Db.Exec("delete from web where id=?", webid)
	Db.Exec("delete from channel where webid=?", webid)
	Db.Exec("delete from article where webid=?", webid)
	deleteDirectory("./templates/" + dir) //删除模板文件
	deleteDirectory("./map/" + dir)       //删除地图相关文件
	updateTemplate()                      //更新模板
	return true
}

// 删除栏目
func Deletechannel(channels []string, webid string) {
	Db.Ping() //测试连接
	//删除栏目
	for _, v := range channels {
		csj := channel.R2(webid, v)
		if csj["topid"] == "0" {
			lschannellb := channellb.R(webid)
			for i, v2 := range lschannellb {
				if v2 == v {
					lschannellb = append(lschannellb[:i], lschannellb[i+1:]...)
				}
			}
			channellb.W(webid, lschannellb) //保存删除后的顶级栏目列表
		} else {
			csj2 := channel.R2(webid, csj["topid"]) //获取顶级栏目
			if csj2 != nil {
				var subid2 = strings.Split(csj2["subid"], ",")
				if subid2[0] != "" {
					for i, v2 := range subid2 {
						if v2 == v {
							subid2 = append(subid2[:i], subid2[i+1:]...)
						}
					}
					csj2["subid"] = strings.Join(subid2, ",")
					channel.W2(webid, csj["topid"], csj2) //保存删除后的顶级栏目
				}
			}
		}
		var subid = strings.Split(csj["subid"], ",")
		if subid[0] != "" {
			Deletechannel(subid, webid)
		}
		var artid = strings.Split(csj["artid"], ",")
		if artid[0] != "" {
			Deletecontent(artid, v, webid)
		}
		//删除缓存
		channel.D2(webid, v)
		Db.Exec("delete from channel where vid=?", v)
	}
}

// 删除文章
func Deletecontent(artid []string, cid, webid string) {
	Db.Ping() //测试连接
	for _, v := range artid {
		delartid(v, cid, webid)
		article.D2(webid, v)
		Db.Exec("delete from article where aid=?", v)
	}
}

// 保存数据
func savedata() {
	log.Println("保存数据")
	Db.Ping() //测试连接
	//获取站点列表
	for id, v := range web.R0() {
		//获取各字段
		b1 := v["webzd"]
		b2 := v["channelzd"]
		b3 := v["articlezd"]
		b4 := v["flinks"]
		b5 := v["zaxiang"]
		b6 := v["post"]
		lschannellb := channellb.R(id)
		b7 := strings.Join(lschannellb, ",") //转字符串
		///Db.Exec("UPDATE web SET webzd = ?, channelzd = ?, articlezd = ?, flinks = ?, zaxiang = ?, post= ? WHERE id = ?", b1, b2, b3, b4, b5, b6, id) //保存到数据库
		Db.Exec(`INSERT INTO web (id,webzd,channelzd,articlezd,flinks,zaxiang,post,channellb) VALUES (?,?,?,?,?,?,?,?) ON DUPLICATE KEY UPDATE webzd=?, channelzd=?, articlezd=?, flinks=?, zaxiang=?, post=?,channellb=?`, id, b1, b2, b3, b4, b5, b6, b7, b1, b2, b3, b4, b5, b6, b7)
	}
	//保存栏目
	for wid, v := range channel.R0() {
		for id, v2 := range v {
			//Db.Exec("UPDATE channel SET shuju = ? WHERE webid=? and vid = ?", v2, wid, id)
			var csj map[string]string
			json.Unmarshal([]byte(v2), &csj)
			topid := csj["topid"]
			Db.Exec(`INSERT INTO channel (webid, vid, topid,shuju) VALUES (?, ?, ?,?) ON DUPLICATE KEY UPDATE shuju=?`, wid, id, topid, v2, v2)
		}
	}
	//保存文章
	for wid, v := range article.R0() {
		for id, v2 := range v {
			Db.Exec(`INSERT INTO article (webid, aid,shuju) VALUES (?, ?,?) ON DUPLICATE KEY UPDATE shuju=?`, wid, id, v2, v2)
			//Db.Exec("UPDATE article SET shuju = ? WHERE webid=? and aid = ?", v2, wid, id)
		}
	}
}

// 保存数据到json
func savedata2() {
	var backsj = Backjson{
		Web:       web.R0(),
		Channel:   channel.R0(),
		Article:   article.R0(),
		Weblb:     weblb.R(),
		Channellb: channellb.R0(),
		Webconfig: webconfig.R0(),
	}
	b, _ := json.Marshal(backsj)
	now := time.Now().Format("2006-01-02_15-04-05")
	backupFile := "./backup/all_" + now + ".json"
	var buf bytes.Buffer
	gz := gzip.NewWriter(&buf) //开启压缩
	defer gz.Close()           // 确保压缩流关闭
	var ok = true
	if _, err := gz.Write(b); err != nil {
		ok = false
	}
	// 确保所有数据都被写入缓冲区
	if err := gz.Flush(); err != nil {
		ok = false
	}
	//关闭压缩
	if err := gz.Close(); err != nil {
		ok = false
	}
	if ok {
		b = buf.Bytes()
		savefile(backupFile, b)
		savefile("./backup/backup.txt", []byte(backupFile))
	}
}

// 保存文件
func savefile(filename string, binaryData []byte) bool {
	// 确保目录存在
	dir := filepath.Dir("./" + filename)
	err := os.MkdirAll(dir, 0755)
	if err != nil {
		return false
	}
	// 打开或创建文件
	file, err := os.Create("./" + filename)
	if err != nil {
		return false
	}
	defer file.Close()
	// 将二进制数据写入文件
	_, err = file.Write([]byte(binaryData))
	if err != nil {
		return false
	}
	// 确保所有数据都被写入文件
	err = file.Sync()
	if err != nil {
		fmt.Println("保存文件失败:", err)
		return false
	}
	return true
}

// 将文章id递归添加到所属目录
func addartid(aid, cid, wid string) {
	//获取栏目信息
	csj := channel.R2(wid, cid)
	if csj["artid"] == "" {
		csj["artid"] = aid
	} else {
		csj["artid"] = csj["artid"] + "," + aid
	}
	if csj["topid"] != "0" {
		addartid(aid, csj["topid"], wid)
	}
	channel.W2(wid, cid, csj)
}

// 将文章id递归删除
func delartid(aid, cid, webid string) {
	//获取栏目信息
	csj := channel.R2(webid, cid)
	if csj["artid"] != "" {
		artid := strings.Split(csj["artid"], ",")
		for i, v := range artid {
			if v == aid {
				artid = append(artid[:i], artid[i+1:]...)
				csj["artid"] = strings.Join(artid, ",")
				channel.W2(webid, cid, csj) //更新栏目数据
				if csj["topid"] != "0" {
					delartid(aid, csj["topid"], webid)
				}
				return
			}
		}
	}
}

// 获取随机字符串
func getrand(length int) string {
	characters := "abcdefghijklmnopqrstuvwxyz0123456789"
	result := make([]byte, length)
	rand.Seed(time.Now().UnixNano())
	for i := 0; i < length; i++ {
		result[i] = characters[rand.Intn(len(characters))]
	}
	return string(result)
}

// 备份数据
func backup_data(wid string) string {
	var backsj = Backjson2{
		Web:       web.R(wid),
		Channellb: channellb.R(wid),
		Channel:   channel.R(wid),
		Article:   article.R(wid),
	}
	//获取站点信息
	lsweb := web.R(wid)
	var sj []Ziduan
	json.Unmarshal([]byte(lsweb["webzd"]), &sj)
	var dir string
	for _, v := range sj {
		if v.K == "dir" {
			dir = v.V
			break
		}
	}
	//获取时间
	now := time.Now().Format("2006-01-02_15-04-05")
	backupFile := "./backup/" + dir + "/" + now + ".json"
	b2, _ := json.Marshal(backsj)
	var buf bytes.Buffer
	gz := gzip.NewWriter(&buf) //开启压缩
	defer gz.Close()           // 确保压缩流关闭
	var ok = true
	if _, err := gz.Write(b2); err != nil {
		ok = false
	}
	// 确保所有数据都被写入缓冲区
	if err := gz.Flush(); err != nil {
		ok = false
	}
	//关闭压缩
	if err := gz.Close(); err != nil {
		ok = false
	}
	if ok {
		b2 = buf.Bytes()
		savefile(backupFile, b2)
		return "新建备份成功！" + now + ".json<br>"
	} else {
		return "备份失败！"
	}
}

// 还原数据
func restore_data(filename, wid string) string {
	Db.Ping()
	Db.Exec("delete from channel where webid=?", wid)
	Db.Exec("delete from article where webid=?", wid)
	data, err := os.ReadFile(filename)
	var ok = true
	if err != nil {
		ok = false
	}
	//解压
	var buf bytes.Buffer
	buf.Write(data)
	gr, err := gzip.NewReader(&buf)
	if err != nil {
		ok = false
	}
	defer gr.Close()
	decompressedContent, err := io.ReadAll(gr)
	if err != nil {
		ok = false
	}
	if ok {
		var backsj Backjson2
		json.Unmarshal(decompressedContent, &backsj)
		//获取原始 "id" "dir" "muban"
		lsweb := web.R(wid)
		var sj []Ziduan
		json.Unmarshal([]byte(lsweb["webzd"]), &sj)
		var dir, url, muban string
		for _, v := range sj {
			if v.K == "dir" {
				dir = v.V
			} else if v.K == "link" {
				url = v.V
			}
		}
		var sj2 map[string]string
		json.Unmarshal([]byte(lsweb["zaxiang"]), &sj2)
		muban = sj2["muban"]
		web.W(wid, backsj.Web)
		channel.W(wid, backsj.Channel)
		article.W(wid, backsj.Article)
		channellb.W(wid, backsj.Channellb)
		lsweb = web.R(wid)
		var sj3 []Ziduan
		json.Unmarshal([]byte(lsweb["webzd"]), &sj3)
		for i, v := range sj3 {
			if v.K == "dir" {
				sj3[i].V = dir
			} else if v.K == "link" {
				sj3[i].V = url
			}
		}
		var sj4 map[string]string
		json.Unmarshal([]byte(lsweb["zaxiang"]), &sj4)
		sj4["muban"] = muban
		b5, _ := json.Marshal(sj3)
		lsweb["webzd"] = string(b5)
		b6, _ := json.Marshal(sj4)
		lsweb["zaxiang"] = string(b6)
		web.W(wid, lsweb) //更新web
		for k, v := range backsj.Channel {
			var csj map[string]string
			json.Unmarshal([]byte(v), &csj)
			Db.Exec("INSERT INTO channel (webid, vid, topid, shuju) VALUES (?,?,?,?)", wid, k, csj["topid"], v)
		}
		for k, v := range backsj.Article {
			Db.Exec("insert into article (webid,aid,shuju) values(?,?,?)", wid, k, v) //执行sql语句
		}
		return "还原数据成功！"
	} else {
		return "还原数据失败！"
	}
}

// 更新MAP
func upmap(wid string) {
	//获取dir link mlink
	var dir, link string
	lsweb := web.R(wid)
	var sj []Ziduan
	json.Unmarshal([]byte(lsweb["webzd"]), &sj)
	for _, v := range sj {
		if v.K == "link" {
			link = v.V
		}
		if v.K == "dir" {
			dir = v.V
		}
	}
	//更新txt
	var txt, xml, html string
	rq := time.Now().Format("2006-01-02")
	txt += link + "\n"
	xml += `<?xml version="1.0" encoding="utf-8"?>\n`
	xml += "<urlset>\n"
	xml += ` <url>\n<loc>` + link + `</loc>\n<lastmod>` + rq + `</lastmod>\n<changefreq>daily</changefreq>\n<priority>1.0</priority>\n</url>\n`
	html += `<!DOCTYPE html><html><head><meta charset="UTF-8"><meta name="viewport" content="width=device-width,initial-scale=1, minimum-scale=1, maximum-scale=1, user-scalable=no" /><title>网站地图</title></head><body>`
	html += `<p>首页：<a href="` + link + `">网站首页</a></p>`
	lschannel := channel.R(wid)
	for i, v := range lschannel {
		var neirong map[string]string
		json.Unmarshal([]byte(v), &neirong)
		txt += link + "/c/" + i + "\n"
		xml += "<url>\n<loc>" + link + "/c/" + i + "</loc>\n<lastmod>" + rq + "</lastmod>\n<changefreq>daily</changefreq>\n<priority>0.9</priority>\n</url>\n"
		html += `<p>目录：<a href="` + link + `/c/` + i + `">` + neirong["title"] + `</a></p>`
	}
	lsarticle := article.R(wid)
	for i, v := range lsarticle {
		var neirong map[string]string
		json.Unmarshal([]byte(v), &neirong)
		txt += link + "/a/" + i + "\n"
		xml += "<url>\n<loc>" + link + "/a" + i + "</loc>\n<lastmod>" + rq + "</lastmod>\n<changefreq>daily</changefreq>\n<priority>0.8</priority>\n</url>\n"
		html += `<p>文章：<a href="` + link + `/c/` + i + `">` + neirong["title"] + `</a></p>`
	}
	xml += "</urlset>\n"
	html += "</body></html>"
	txt = strings.Replace(txt, "\\n", "\n", -1)
	xml = strings.Replace(xml, "\\n", "\n", -1)
	savefile("./map/"+dir+"/sitemap.xml", []byte(xml))
	savefile("./map/"+dir+"/sitemap.txt", []byte(txt))
	savefile("./map/"+dir+"/sitemap.html", []byte(html))
	//更新XML
	txt = ""
	txt += "User-agent: * \n"
	txt += "Allow: / \n"
	txt += "Sitemap: " + link + "/sitemap.xml \n"
	txt += "Sitemap: " + link + "/sitemap.txt \n"
	txt = strings.Replace(txt, "\\n", "\n", -1)
	savefile("./map/"+dir+"/robots.txt", []byte(txt))
}

// 判断字符串是否在切片中
func existsInSlice(slice []string, value string) bool {
	for _, item := range slice {
		if item == value {
			return true
		}
	}
	return false
}

// 判断字符串是否包含在切片中 关键词 搜索用
func containsValue(str string, keys []string) int {
	i := 0
	for _, value := range keys {
		if strings.Contains(str, value) {
			i++
		}
	}
	return i
}

// 将搜索结果关键词加红显示
func addtags(str string, values []string) string {
	for _, value := range values {
		str = strings.ReplaceAll(str, value, `<span style="color:red;">`+value+`</span>`)
	}
	return str
}

// 获取切片指定值的前一个和后一个值
func getPrevAndNext(slice []string, target string) (string, string) {
	var prev, next string
	for i, v := range slice {
		if v == target {
			if i > 0 {
				prev = slice[i-1]
			} else {
				prev = ""
			}
			if i < len(slice)-1 {
				next = slice[i+1]
			} else {
				next = ""
			}
			return prev, next
		}
	}
	return "", "" // 如果未找到目标值，返回空 表示不存在
}

// 获取导航 wid网站id row数量 noid排除id muban顶级栏目模板 muban2二级栏目模板
func getnavigation(wid, row, noid, muban, muban2 string) template.HTML {
	num, ok := strconv.Atoi(row)
	if ok != nil {
		num = 100
	}
	noids := strings.Split(noid, ",")
	var neirong string = ""
	daohang := channellb.R(wid)
	if len(daohang) > 0 {
		i := 0
		for _, v5 := range daohang {
			if i >= num {
				break
			}
			if existsInSlice(noids, v5) {
				continue
			}
			csj := channel.R2(wid, v5)
			mb := ""
			mb = strings.ReplaceAll(muban, "[c_id]", v5)
			for k, v2 := range csj {
				mb = strings.ReplaceAll(mb, "[c_"+k+"]", v2)
			}
			mb2 := ""
			erjidaohang := strings.Split(csj["subid"], ",") //转切片
			if len(erjidaohang) > 0 && erjidaohang[0] != "" {
				for _, v6 := range erjidaohang {
					csj2 := channel.R2(wid, v6)
					mb3 := strings.ReplaceAll(muban2, "[c_id]", v6)
					for k, v2 := range csj2 {
						mb3 = strings.ReplaceAll(mb3, "[c_"+k+"]", v2)
					}
					mb2 += mb3
				}
			}
			mb = strings.ReplaceAll(mb, "{erji}", mb2)
			neirong += mb
			i++

		}
	}
	return template.HTML(neirong)
}

// 获取文章列表 wid网站id cid栏目id row数量 yema页码 muban顶级栏目模板
func getarticle(wid, cid, row, yema, muban string) Artlist {
	num, ok := strconv.Atoi(row)
	if ok != nil {
		num = 20
	}
	yemai, err := strconv.Atoi(yema)
	if err != nil {
		yemai = 1
	}
	if yemai < 1 {
		yemai = 1
	}
	ksid := (yemai - 1) * num
	jsid := ksid + num
	i := 0
	var neirong string = ""
	var neirong2 string = ""
	if cid == "" {
		for id, v2 := range article.R(wid) {
			if i >= num {
				break
			}
			var asj map[string]string
			json.Unmarshal([]byte(v2), &asj)
			y, m, d, h, m2, s := timefenjie(asj["time"])
			mb := strings.ReplaceAll(muban, "[a_id]", id)
			mb = strings.ReplaceAll(mb, "[y]", y)
			mb = strings.ReplaceAll(mb, "[m]", m)
			mb = strings.ReplaceAll(mb, "[d]", d)
			mb = strings.ReplaceAll(mb, "[h]", h)
			mb = strings.ReplaceAll(mb, "[i]", m2)
			mb = strings.ReplaceAll(mb, "[s]", s)
			for k, v3 := range asj {
				mb = strings.ReplaceAll(mb, "[a_"+k+"]", v3)
			}
			neirong += mb
			i++
		}
	} else {
		csj := channel.R2(wid, cid)
		artids := strings.Split(csj["artid"], ",") //转切片
		zsl := len(artids)
		if zsl > 0 && artids[0] != "" && ksid < zsl && jsid <= zsl {
			for _, id := range artids[ksid:jsid] {
				if i >= num {
					break
				}
				asj := article.R2(wid, id)
				y, m, d, h, m2, s := timefenjie(asj["time"])
				mb := strings.ReplaceAll(muban, "[a_id]", id)
				mb = strings.ReplaceAll(mb, "[y]", y)
				mb = strings.ReplaceAll(mb, "[m]", m)
				mb = strings.ReplaceAll(mb, "[d]", d)
				mb = strings.ReplaceAll(mb, "[h]", h)
				mb = strings.ReplaceAll(mb, "[i]", m2)
				mb = strings.ReplaceAll(mb, "[s]", s)
				for k, v3 := range asj {
					mb = strings.ReplaceAll(mb, "[a_"+k+"]", v3)
				}
				neirong += mb
				i++
			}
			yes := math.Ceil(float64(zsl) / float64(num))
			if yes > 1 {
				for y := 1; y <= int(yes); y++ {
					if y == yemai {
						neirong2 += `<a href="/c/` + cid + `_` + strconv.Itoa(y) + `"><b>` + strconv.Itoa(y) + `</b></a>`
					} else {
						neirong2 += `<a href="/c/` + cid + `_` + strconv.Itoa(y) + `">` + strconv.Itoa(y) + `</a>`
					}
				}
			}
		} else {
			neirong = "暂无相关文章"
		}
	}
	return Artlist{
		List: template.HTML(neirong),
		Page: template.HTML(neirong2),
	}
}

// 获取友情连接
func getflink(wid, row, muban string) template.HTML {
	num, ok := strconv.Atoi(row)
	if ok != nil {
		num = 20
	}
	i := 0
	var neirong string = ""
	lsweb := web.R(wid)
	var flinks []FriendLink
	json.Unmarshal([]byte(lsweb["flinks"]), &flinks)
	for _, v2 := range flinks {
		if i >= num {
			break
		}
		mb := strings.ReplaceAll(muban, "[title]", v2.N)
		mb = strings.ReplaceAll(mb, "[link]", v2.U)
		neirong += mb
		i++
	}
	return template.HTML(neirong)
}

// 获取指定栏目title
func getchanneltitle(wid, cid string) string {
	return channel.R2(wid, cid)["title"]
}

// 获取相关栏目wid网站id cid栏目id row数量 noid排除id muban顶级栏目模板
func getgxlanmu(wid, cid, row, noid, muban string) template.HTML {
	var neirong string
	num, ok := strconv.Atoi(row)
	if ok != nil {
		num = 100
	}
	noids := strings.Split(noid, ",")
	i := 0
	csj := channel.R2(wid, cid)
	var strsubid string
	if csj["topid"] == "0" {
		strsubid = csj["subid"]
	} else {
		if csj["subid"] != "" {
			strsubid = csj["subid"]
		} else {
			csj := channel.R2(wid, csj["topid"])
			strsubid = csj["subid"]
		}
	}
	if strsubid != "" {
		lanmus := strings.Split(strsubid, ",") //转切片
		if len(lanmus) > 0 && lanmus[0] != "" {
			for _, lanmu := range lanmus {
				if i >= num {
					break
				}
				if existsInSlice(noids, lanmu) {
					continue
				}
				csj := channel.R2(wid, lanmu)
				mb := strings.ReplaceAll(muban, "[c_id]", lanmu)
				for k, v := range csj {
					mb = strings.ReplaceAll(mb, "[c_"+k+"]", v)
				}
				neirong += mb
				i++
			}
		}
	}
	return template.HTML(neirong)
}

// 获取搜索列表
func getsearchlist(wid, key, row, yema, muban string) Artlist {
	keys := strings.Split(key, ",") //转切片
	num, ok := strconv.Atoi(row)
	if ok != nil {
		num = 20
	}
	yemai, err := strconv.Atoi(yema)
	if err != nil {
		yemai = 1
	}
	if yemai < 1 {
		yemai = 1
	}
	ksid := (yemai - 1) * num
	jsid := ksid + num
	i := 0
	var neirong string = ""
	var neirong2 string = ""
	list := []Searchlist{}
	listmap := make(map[string]map[string]string)
	for id, v2 := range article.R(wid) {
		var asj map[string]string
		json.Unmarshal([]byte(v2), &asj)
		nums := containsValue(asj["title"], keys) + containsValue(asj["keywords"], keys) + containsValue(asj["description"], keys)
		if nums > 0 {
			listmap[id] = asj
			list = append(list, Searchlist{I: id, N: nums})
		}
	}
	zsl := len(list)
	if zsl > 0 && ksid < zsl && jsid <= zsl {
		// 按值排序
		sort.Slice(list, func(i, j int) bool {
			return list[i].N > list[j].N
		})
		//历遍
		for _, v := range list[ksid:jsid] {
			if i >= num {
				break
			}
			asj := listmap[v.I]
			y, m, d, h, m2, s := timefenjie(asj["time"])
			mb := strings.ReplaceAll(muban, "[a_id]", v.I)
			mb = strings.ReplaceAll(mb, "[y]", y)
			mb = strings.ReplaceAll(mb, "[m]", m)
			mb = strings.ReplaceAll(mb, "[d]", d)
			mb = strings.ReplaceAll(mb, "[h]", h)
			mb = strings.ReplaceAll(mb, "[i]", m2)
			mb = strings.ReplaceAll(mb, "[s]", s)
			for k, v3 := range asj {
				if k == "title" || k == "keywords" || k == "description" {
					mb = strings.ReplaceAll(mb, "[a_"+k+"]", addtags(v3, keys)) //加红
					continue
				}
				mb = strings.ReplaceAll(mb, "[a_"+k+"]", v3)
			}
			neirong += mb
			i++
		}
		yes := math.Ceil(float64(zsl) / float64(num))
		if yes > 1 {
			for y := 1; y <= int(yes); y++ {
				if y == yemai {
					neirong2 += `<a href="/s/` + key + `_` + strconv.Itoa(y) + `"><b>` + strconv.Itoa(y) + `</b></a>`
				} else {
					neirong2 += `<a href="/c/` + key + `_` + strconv.Itoa(y) + `">` + strconv.Itoa(y) + `</a>`
				}
			}
		}
	} else {
		neirong = "没有相关内容"
	}
	return Artlist{
		List: template.HTML(neirong),
		Page: template.HTML(neirong2),
	}
}

// 将字符串转为html
func tohtml(str string) template.HTML {
	return template.HTML(str)
}

// 时间转换
func totime(str, muban string) string {
	t, _ := time.Parse("2006-01-02 15:04:05.999999999 -0700 MST", str)
	return t.Format(muban)
}

// 时间分解
func timefenjie(str string) (string, string, string, string, string, string) {
	t, _ := time.Parse("2006-01-02 15:04:05.999999999 -0700 MST", str)
	return t.Format("2006"), t.Format("01"), t.Format("02"), t.Format("15"), t.Format("04"), t.Format("05")
}

// 获取网站信息
func get_info(wid string) map[string]string {
	lsweb := web.R(wid)
	if lsweb == nil {
		return nil
	}
	var sj2 = make(map[string]string)
	var sj []Ziduan
	json.Unmarshal([]byte(lsweb["webzd"]), &sj)
	for _, v := range sj {
		sj2["w_"+v.K] = v.V
	}
	sj2["w_id"] = wid
	return sj2
}

// 获取栏目信息
func get_list(wid, cid2 string) map[string]string {
	qp := strings.Split(cid2, "_") //转切片
	if qp[0] == "" {
		return nil
	}
	cid := qp[0]
	yema := 1

	if len(qp) > 1 && qp[1] != "" {
		yema2, err := strconv.Atoi(qp[1])
		if err == nil {
			yema = yema2
		}
	}
	csj := channel.R2(wid, cid)
	if csj == nil {
		return nil
	}
	var sj2 = make(map[string]string)
	for k, v2 := range csj {
		sj2["c_"+k] = html.UnescapeString(v2)
	}
	sj2["c_id"] = cid
	sj2["c_yema"] = strconv.Itoa(yema)
	if yema > 1 {
		sj2["c_yema2"] = ` 第` + strconv.Itoa(yema) + `页`
	} else {
		sj2["c_yema2"] = ``
	}
	//获取网站数据
	lsweb := web.R(wid)
	var sj []Ziduan
	json.Unmarshal([]byte(lsweb["webzd"]), &sj)
	for _, v := range sj {
		sj2["w_"+v.K] = v.V
	}
	//获取当前位置
	sj2["position"] = ""
	sj2["position"] += `<a href="` + sj2["w_link"] + `">` + sj2["w_homename"] + `</a> > `
	if csj["topid"] == "0" {
		sj2["position"] += `<a href="` + sj2["w_link"] + `/c/` + cid + `">` + sj2["c_title"] + `</a> >`
	} else {
		csj5 := channel.R2(wid, sj2["c_topid"])
		sj2["position"] += `<a href="` + sj2["w_link"] + `/c/` + csj["topid"] + `">` + csj5["title"] + `</a> > `
		sj2["position"] += `<a href="` + sj2["w_link"] + `/c/` + cid + `">` + sj2["c_title"] + `</a> >`
	}
	sj2["w_id"] = wid
	return sj2
}

// 获取文章信息
func get_art(wid, aid string) map[string]string {
	asj := article.R2(wid, aid)
	if asj == nil {
		return nil
	}
	var sj2 = make(map[string]string)
	looks, err := strconv.Atoi(asj["looks"])
	if err != nil {
		looks = 0
	}
	looks++
	asj["looks"] = strconv.Itoa(looks)
	article.W2(wid, aid, asj) //更新浏览量
	for k, v2 := range asj {
		sj2["a_"+k] = html.UnescapeString(v2)
	}
	sj2["y"], sj2["m"], sj2["d"], sj2["h"], sj2["i"], sj2["s"] = timefenjie(asj["time"])
	sj2["a_id"] = aid
	sj2["c_id"] = asj["topid"]
	//获取所属栏目数据
	csj := channel.R2(wid, asj["topid"])
	for k, v4 := range csj {
		sj2["c_"+k] = html.UnescapeString(v4)
	}
	artids := strings.Split(csj["artid"], ",") //转切片
	previd, nextid := getPrevAndNext(artids, aid)
	sj2["previd"] = ""
	sj2["prevtitle"] = ""
	sj2["prev"] = `上一篇:没有了`
	sj2["nextid"] = ""
	sj2["nexttitle"] = ""
	sj2["next"] = `下一篇:没有了`
	if previd != "" {
		lsasj := article.R2(wid, previd)
		if lsasj != nil {
			sj2["previd"] = previd
			sj2["prevtitle"] = lsasj["title"]
			sj2["prev"] = `上一篇:<a href="/a/` + previd + `">` + lsasj["title"] + `</a>`
		}
	}
	if nextid != "" {
		lsasj := article.R2(wid, nextid)
		if lsasj != nil {
			sj2["nextid"] = nextid
			sj2["nexttitle"] = lsasj["title"]
			sj2["next"] = `下一篇:<a href="/a/` + nextid + `">` + lsasj["title"] + `</a>`
		}
	}
	//获取网站数据
	lsweb := web.R(wid)
	var sj []Ziduan
	json.Unmarshal([]byte(lsweb["webzd"]), &sj)
	for _, v := range sj {
		sj2["w_"+v.K] = v.V
	}
	//获取当前位置
	sj2["position"] = ""
	sj2["position"] += `<a href="` + sj2["w_link"] + `">` + sj2["w_homename"] + `</a> > `
	if sj2["c_topid"] == "0" {
		sj2["position"] += `<a href="` + sj2["w_link"] + `/c/` + asj["topid"] + `">` + sj2["c_title"] + `</a> >`
	} else {
		csj5 := channel.R2(wid, sj2["c_topid"])
		sj2["position"] += `<a href="` + sj2["w_link"] + `/c/` + sj2["c_topid"] + `">` + csj5["title"] + `</a> > `
		sj2["position"] += `<a href="` + sj2["w_link"] + `/c/` + asj["topid"] + `">` + sj2["c_title"] + `</a> >`
	}
	sj2["w_id"] = wid
	return sj2
}

// 获取远程网址内容
func fetchHttpsContent(url string) string {
	client := &http.Client{
		Timeout: 10 * time.Second, // 设置 10 秒超时
	}
	req, err := http.NewRequest("GET", url, nil)
	if err != nil {
		return ""
	}
	req.Header.Set("User-Agent", "YiBaoCMS") // 设置 User-Agent
	resp, err := client.Do(req)
	if err != nil {
		return ""
	}
	defer resp.Body.Close()
	body, err := io.ReadAll(resp.Body)
	if err != nil {
		return ""
	}
	enc, _, _ := charset.DetermineEncoding(body, resp.Header.Get("Content-Type"))           // 获取编码
	if strings.Contains(fmt.Sprintf("%v", enc), fmt.Sprintf("%v", simplifiedchinese.GBK)) { // 判断是否为 GBK 编码
		reader := transform.NewReader(bytes.NewReader(body), enc.NewDecoder())
		decodedBody, err := io.ReadAll(reader)
		if err != nil {
			return string(body)
		}
		return string(decodedBody)
	}
	return string(body)
}

// 获取css js文件
func modifylinks(htmlContent, h2, h4, stdir, mbn string, caijiurled map[string]string) (string, map[string]string) {
	var title, keywords, description, canonical string
	if mbn == "index.htm" || mbn == "index_m.htm" {
		title = "{{.w_title}}"
		keywords = "{{.w_keywords}}"
		description = "{{.w_description}}"
		canonical = "{{.w_link}}"
	} else if mbn == "c_index.htm" || mbn == "c_images.htm" || mbn == "c_list.htm" || mbn == "c_index_m.htm" || mbn == "c_images_m.htm" || mbn == "c_list_m.htm" {
		title = "{{.c_seotitle}}-{{.w_ftitle}}{{.c_yema2}}"
		keywords = "{{.c_keywords}}"
		description = "{{.c_description}}"
		canonical = "{{.w_link}}/c/{{.c_id}}"
	} else if mbn == "a_images.htm" || mbn == "a_article.htm" || mbn == "a_images_m.htm" || mbn == "a_article_m.htm" {
		title = "{{.a_seotitle}}-{{.w_ftitle}}"
		keywords = "{{.a_keywords}}"
		description = "{{.a_description}}"
		canonical = "{{.w_link}}/a/{{.a_id}}"
	} else if mbn == "search.htm" || mbn == "search_m.htm" {
		title = "{{.s_key}}-{{.w_ftitle}}{{.s_yema2}}"
		keywords = "{{.s_key}}"
		description = "{{.s_key}}"
		canonical = "{{.w_link}}/c/{{.s_key}}"
	}
	doc, err := html.Parse(strings.NewReader(htmlContent))
	if err != nil {
		log.Fatal(err)
	}
	var f func(*html.Node)
	f = func(n *html.Node) {
		if n.Type == html.ElementNode && (n.Data == "a" || n.Data == "link" || n.Data == "script" || n.Data == "img" || n.Data == "meta") {
			for i, attr := range n.Attr {
				if n.Data == "a" {
					for i, a := range n.Attr {
						if a.Key == "href" {
							n.Attr[i].Val = ""
						}
					}
				} else if n.Data == "link" || n.Data == "script" {
					if attr.Key == "rel" {
						if attr.Val == "canonical" {
							for i, a := range n.Attr {
								if a.Key == "href" {
									n.Attr[i].Val = canonical
								}
							}
						} else if attr.Val == "archives" {
							for i, a := range n.Attr {
								if a.Key == "href" {
									n.Attr[i].Val = "{{.w_link}}/sitemap.xml"
								} else if a.Key == "title" {
									n.Attr[i].Val = "{{.w_title}}"
								}
							}
						} else if strings.Contains(attr.Val, "Icon") || strings.Contains(attr.Val, "icon") {
							for i, a := range n.Attr {
								if a.Key == "href" {
									n.Attr[i].Val = "/favicon.ico"
								}
							}
						}
					}
					if (attr.Key == "href" && strings.HasSuffix(attr.Val, ".css")) || (attr.Key == "src" && strings.HasSuffix(attr.Val, ".js")) {
						var cssurl, newcssurl string
						oldcssurl := attr.Val
						s, h, p, _ := parseURL(oldcssurl)
						if s == "" {
							if h == "" {
								if p == "" {
									cssurl = ""
								} else {
									if string(p[0]) == "." {
										cssurl = h2 + p
									} else if string(p[0]) == "/" {
										cssurl = h4 + p
									} else {
										cssurl = h4 + "/" + p
									}
								}
							} else {
								cssurl = h4 + "/" + p
							}
						} else {
							cssurl = oldcssurl
						}
						if cssurl != "" && caijiurled[cssurl] == "" {
							neirong := fetchHttpsContent(cssurl)
							if neirong != "" {
								name := oldcssurl[strings.LastIndex(oldcssurl, "/")+1:]
								nh2 := strings.ReplaceAll(cssurl, name, "")
								path := stdir + "/" + name // 文件保存地址
								newcssurl = path[1:]       // 去掉开头的 .
								neirong, caijiurled = modifyAndReturnUrls(neirong, nh2, h4, stdir, caijiurled)
								savefile(path, []byte(neirong)) //	保存文件
								caijiurled[cssurl] = newcssurl  //添加到caijiurled
							}
						}
						n.Attr[i].Val = caijiurled[cssurl]
					}
				} else if n.Data == "meta" {
					if attr.Key == "name" {
						if attr.Val == "keywords" {
							for i, a := range n.Attr {
								if a.Key == "content" {
									n.Attr[i].Val = keywords
								}
							}
						} else if attr.Val == "description" {
							for i, a := range n.Attr {
								if a.Key == "content" {
									n.Attr[i].Val = description
								}
							}
						} else if attr.Val == "author" {
							for i, a := range n.Attr {
								if a.Key == "content" {
									n.Attr[i].Val = "YiBaoCMS"
								}
							}
						} else if attr.Val == "page-topic" {
							for i, a := range n.Attr {
								if a.Key == "content" {
									n.Attr[i].Val = "{{.w_title}}"
								}
							}
						} else if attr.Val == "copyright" {
							for i, a := range n.Attr {
								if a.Key == "content" {
									n.Attr[i].Val = "Copyright {{.w_title}}"
								}
							}
						} else if attr.Val == "apple-mobile-web-app-title" || attr.Val == "msapplication-TileTitle" || attr.Val == "application-name" || attr.Val == "msapplication-tooltip" {
							for i, a := range n.Attr {
								if a.Key == "content" {
									n.Attr[i].Val = "{{.w_ftitle}}"
								}
							}
						} else if attr.Val == "msapplication-starturl" {
							for i, a := range n.Attr {
								if a.Key == "content" {
									n.Attr[i].Val = "{{.w_link}}"
								}
							}
						} else if attr.Val == "applicable-device" {
							for i, a := range n.Attr {
								if a.Key == "content" {
									n.Attr[i].Val = "pc,mobile"
								}
							}
						} else if attr.Val == "mobile-agent" {
							for i, a := range n.Attr {
								if a.Key == "content" {
									n.Attr[i].Val = "format=html5; url=" + canonical
								}
							}
						}
					} else if attr.Key == "http-equiv" {
						if attr.Val == "Content-Type" {
							for i, a := range n.Attr {
								if a.Key == "content" {
									n.Attr[i].Val = "text/html;charset=utf-8"
								}
							}
						} else if attr.Val == "mobile-agent" {
							for i, a := range n.Attr {
								if a.Key == "content" {
									n.Attr[i].Val = "format=html5; url=" + canonical
								}
							}
						}

					}
				} else if n.Data == "img" && attr.Key == "src" {
					oldpath := attr.Val
					var imgurl, newpath string
					s, h, p, _ := parseURL(oldpath)
					if s == "" {
						if h == "" {
							if p == "" {
								imgurl = ""
							} else {
								if string(p[0]) == "." {
									imgurl = h2 + p
								} else if string(p[0]) == "/" {
									imgurl = h4 + p
								} else {
									imgurl = h4 + "/" + p
								}
							}
						} else {
							imgurl = h4 + "/" + p
						}
					} else {
						imgurl = oldpath
					}
					if imgurl != "" && caijiurled[imgurl] == "" {
						neirong := fetchHttpsContent(imgurl)
						if neirong != "" {
							name := imgurl[strings.LastIndex(imgurl, "/")+1:]
							path := stdir + "/static/" + name // 文件保存地址
							newpath = path[1:]                // 去掉开头的 .
							savefile(path, []byte(neirong))   //	保存文件
							caijiurled[imgurl] = newpath      //添加到caijiurled
						}
					}
					n.Attr[i].Val = caijiurled[imgurl]
				}

			}
		} else if n.Type == html.ElementNode && n.Data == "title" {
			// 找到 <title> 标签的子节点（即文本节点）
			for c := n.FirstChild; c != nil; c = c.NextSibling {
				if c.Type == html.TextNode {
					c.Data = title
				}
			}
		}
		for c := n.FirstChild; c != nil; c = c.NextSibling {
			f(c)
		}
	}
	f(doc)
	var newHTML strings.Builder
	html.Render(&newHTML, doc)
	return newHTML.String(), caijiurled
}

// 获取内容中的url()
func modifyAndReturnUrls(neirong, h2, h4, stdir string, caijiurled map[string]string) (string, map[string]string) {
	re := regexp.MustCompile(`url\(['"]?(.*?)['"]?\)`)
	matches := re.FindAllStringSubmatch(neirong, -1)
	for _, match := range matches {
		if len(match) > 1 {
			oldpath := match[1]
			if strings.Contains(oldpath, "data:") || strings.Contains(oldpath, "+") || strings.Contains(oldpath, "#") {
				continue
			}
			var url, newpath string
			s, h, p, _ := parseURL(oldpath)
			if s == "" {
				if h == "" {
					if p == "" {
						url = ""
					} else {
						if string(p[0]) == "." {
							url = h2 + p
						} else if string(p[0]) == "/" {
							url = h4 + p
						} else {
							url = h4 + "/" + p
						}
					}
				} else {
					url = h4 + "/" + p
				}
			} else {
				url = oldpath
			}
			if url != "" && caijiurled[url] == "" {
				b := fetchHttpsContent(url)
				if b != "" {
					name := oldpath[strings.LastIndex(oldpath, "/")+1:]
					newurl := stdir + "/static/" + name // 文件保存地址
					newpath = newurl[1:]                // 去掉开头的 .
					savefile(newurl, []byte(b))         //保存文件
					caijiurled[url] = newpath           //添加到caijiurled
				}
			}
			neirong = strings.ReplaceAll(neirong, oldpath, caijiurled[url])
		}
	}
	return neirong, caijiurled
}

// 开始采集
func caijistart(caijiurl map[string]map[string]string, dir string, conn *Conn) {
	time := time.Now().Format("2006-01-02_15-04-05")
	mbdir := "./templates/" + dir + "/" + time //模板路径
	stdir := "./static/" + dir + "/" + time    //静态文件路径
	msg := map[string]string{"infos": "开始采集..."}
	send(conn, msg)
	var caijiurled = map[string]string{} //已经采集的网址
	for _, caijiurl2 := range caijiurl {
		for mbn, h := range caijiurl2 {
			if h != "" {
				//h网址 h2网址带目录部分 h3域名 h4协议带域名
				msg := map[string]string{"infos": "开始采集:" + h}
				send(conn, msg)
				scheme, host, path, _ := parseURL(h)
				h2 := scheme + "://" + host + path[:strings.LastIndex(path, "/")+1] //h2网址带目录部分
				//h3 := host                                                          //域名
				h4 := scheme + "://" + host         //h4协议带域名
				htmlContent := fetchHttpsContent(h) // 获取页面内容
				if htmlContent == "" {
					msg := map[string]string{"infos": h + " 采集失败！"}
					send(conn, msg)
				} else {
					msg = map[string]string{"infos": h + " 模板采集成功！<br>开始采集css js img文件...."}
					send(conn, msg)
					htmlContent, caijiurled = modifyAndReturnUrls(htmlContent, h2, h4, stdir, caijiurled) //修改url()
					htmlContent, caijiurled = modifylinks(htmlContent, h2, h4, stdir, mbn, caijiurled)    // 修改 获取css js
					msg = map[string]string{"infos": h + " 模板采集完成！"}
					send(conn, msg)
				}
				newdata := `{{define "` + dir + `/` + time + `/` + mbn + `"}}<!--模板标识 请勿删除这行-->\n`
				newdata += htmlContent
				newdata += `\n{{end}}<!--模板标识结束 请勿删除这行-->`
				newdata = strings.Replace(newdata, "\\n", "\n", -1)
				savefile(mbdir+"/"+mbn, []byte(newdata)) //保存模板文件
			}
		}
	}
	//创建空的head.htm和footer.htm
	newdata := `{{define "` + dir + `/` + time + `/head.htm"}}<!--模板标识 请勿删除这行 在需要的地方使用  {{template "` + dir + `/` + time + `/head.htm".}} 嵌套调用 -->\n\n`
	newdata += `\n{{end}}<!--模板标识结束 请勿删除这行-->`
	newdata = strings.Replace(newdata, "\\n", "\n", -1)
	savefile(mbdir+"/head.htm", []byte(newdata)) //保存模板文件
	newdata = `{{define "` + dir + `/` + time + `/footer.htm"}}<!--模板标识 请勿删除这行 在需要的地方使用  {{template "` + dir + `/` + time + `/footer.htm".}} 嵌套调用-->\n\n`
	newdata += `\n{{end}}<!--模板标识结束 请勿删除这行-->`
	newdata = strings.Replace(newdata, "\\n", "\n", -1)
	savefile(mbdir+"/footer.htm", []byte(newdata)) //保存模板文件
}
