package main

import (
	"html/template"
	"sync"

	"github.com/gorilla/websocket"
)

// web
type WebData struct {
	data map[string]map[string]string
	lock sync.RWMutex
}

// channel
type ChannelData struct {
	data map[string]map[string]string
	lock sync.RWMutex
}

// article
type ArticleData struct {
	data map[string]map[string]string
	lock sync.RWMutex
}

// user
type UserData struct {
	data map[string]*Conn
	lock sync.RWMutex
}

// channellb
type ChannellbData struct {
	data map[string][]string
	lock sync.RWMutex
}

// webconfig
type WebconfigData struct {
	data map[string]map[string]string
	lock sync.RWMutex
}

// ipjilu
type IpjiluData struct {
	data map[string]int64
	lock sync.RWMutex
}

// weblb
type WeblbData struct {
	data []Weblb
	lock sync.RWMutex
}

// websocket
type Conn struct {
	*websocket.Conn `json:"conn"`
	Uid             string `json:"uid"`
	Wid             string `json:"wid"`
}

// 网站列表
type Weblb struct {
	I string `json:"i"`
	U string `json:"u"`
}

// 接收的数据
type Data struct {
	N       string                       `json:"n"`
	P       string                       `json:"p"`
	Cmd     string                       `json:"cmd"`
	Sid     string                       `json:"s"`
	Cid     int                          `json:"c"`
	Shuru   string                       `json:"m"`
	Shuju   map[string]string            `json:"sj"`
	Ping    string                       `json:"ping"`
	Imgtp   string                       `json:"imgtp"`
	Imgs    map[string]map[string]string `json:"imgs"`
	Up      string                       `json:"up"`
	Ab      string                       `json:"ab"`
	Ups     string                       `json:"ups"`
	Lookimg string                       `json:"lookimg"`
	Yema    int                          `json:"yema"`
}
type Shuju struct {
	Addziduan  string `json:"addziduan"`
	Addziduan2 string `json:"addziduan2"`
}

// 字段 网站 栏目 文章通用字段
type Ziduan struct {
	N string `json:"n"`
	K string `json:"k"`
	V string `json:"v"`
	T string `json:"t"`
}

// 友情连接
type FriendLink struct {
	N string `json:"n"`
	U string `json:"u"`
}

// 获取文章返回字段
type Artlist struct {
	List template.HTML
	Page template.HTML
}

// 网站列表
type Searchlist struct {
	I string
	N int
}

// 保存json文件
type Backjson struct {
	Web       map[string]map[string]string `json:"web"`
	Channel   map[string]map[string]string `json:"channel"`
	Article   map[string]map[string]string `json:"article"`
	Weblb     []Weblb                      `json:"weblb"`
	Channellb map[string][]string          `json:"channellb"`
	Webconfig map[string]map[string]string `json:"webconfig"`
}

// 备份json文件
type Backjson2 struct {
	Web       map[string]string `json:"web"`
	Channellb []string          `json:"channellb"`
	Channel   map[string]string `json:"channel"`
	Article   map[string]string `json:"article"`
}
