package main

import (
	"database/sql"
	"encoding/json"
	"fmt"
	"html"
	"html/template"
	"log"
	"net/http"
	"os"
	"os/signal"
	"strconv"
	"strings"
	"syscall"
	"time"

	"github.com/gin-gonic/gin"
	_ "github.com/go-sql-driver/mysql"
	"github.com/gorilla/websocket"
)

// 全局变量
var port, dbhost, dbport, dbuser, dbpass, dbname, safeurl, safeuser, safepass, dsn string
var Db *sql.DB  // 初始化数据库连接池
var Db1 *sql.DB // 初始化数据库连接池
var webname string = "怡宝CMS"
var upgrader = websocket.Upgrader{
	ReadBufferSize:  1024,
	WriteBufferSize: 1024,
	CheckOrigin:     func(r *http.Request) bool { return true }, // 允许所有来源的请求
}

var web = &WebData{
	data: make(map[string]map[string]string),
}
var channel = &ChannelData{
	data: make(map[string]map[string]string),
}
var article = &ArticleData{
	data: make(map[string]map[string]string),
}
var user = &UserData{
	data: make(map[string]*Conn),
}
var channellb = &ChannellbData{
	data: make(map[string][]string),
}
var webconfig = &WebconfigData{
	data: make(map[string]map[string]string),
}
var ipjilu = &IpjiluData{
	data: make(map[string]int64),
}
var weblb = &WeblbData{
	data: []Weblb{},
}

// var r = gin.Default()                              // 创建一个默认的路由实例
var r = gin.New() // 创建一个路由实例 没有日志
var jincheng = 1  //用户全局判断当前程序状态 1为启动 2为运行时
func main() {
	// 设置信号捕获
	c := make(chan os.Signal, 1)
	signal.Notify(c, syscall.SIGINT, syscall.SIGTERM)
	// 在一个新的 goroutine 中等待信号
	go func() {
		<-c            // 阻塞直到接收到信号
		if checkdb() { //检查数据库
			savedata() //保存数据到数据库
		} else {
			savedata2() //数据库出错 将数据保存到文件
		}
		Db1.Close() // 关闭数据库连接
		Db.Close()  // 关闭数据库连接
		os.Exit(0)  // 退出程序
	}()
	defer func() {
		if r := recover(); r != nil {
			log.Println("致命异常:", r)
			if jincheng == 2 { //运行期间 异常退出
				if checkdb() { //检查数据库
					savedata() //保存数据到数据库
				} else {
					savedata2() //数据库出错 将数据保存到文件
				}
			}
			os.Exit(0) // 退出程序
		}
		Db1.Close() // 关闭数据库连接
		Db.Close()  // 关闭数据库连接
	}()
	chushihua()                                          //初始化
	jincheng = 2                                         //用户全局判断当前程序状态 1为启动 2为运行时
	gin.SetMode(gin.ReleaseMode)                         // 设置为生产模式
	r.Static("/static", "./static")                      // 设置静态文件目录
	r.Static("/css", "./css")                            // 设置静态文件目录
	r.Static("/js", "./js")                              // 设置静态文件目录
	r.Static("/map", "./map")                            // 设置静态文件目录
	r.StaticFile("/favicon.ico", "./static/favicon.ico") // 设置单个静态文件
	updateTemplate()                                     //更新模板
	//首页
	r.GET("/", func(c *gin.Context) {
		host := c.Request.Host
		//获取网站id
		sj, _ := webconfig.R(host)
		id := sj["id"]
		dir := sj["dir"]
		muban := sj["muban"]
		if id != "" && dir != "" && muban != "" {
			sj := get_info(id) //网站数据
			if sj == nil {
				c.HTML(http.StatusNotFound, "404.htm", gin.H{"title": "404 - " + webname})
			} else {
				//判断是否开启移动端
				var mubanname string = "index.htm"
				if sj["w_wap"] == "1" {
					userAgent := c.Request.Header.Get("User-Agent")
					if strings.Contains(userAgent, "Mobile") || strings.Contains(userAgent, "Android") || strings.Contains(userAgent, "iPhone") {
						mubanname = strings.Replace(mubanname, ".htm", "_m.htm", 1)
					}
				}
				c.HTML(http.StatusOK, dir+"/"+muban+"/"+mubanname, sj)
			}
		} else {
			c.HTML(http.StatusNotFound, "404.htm", gin.H{"title": "404 - " + webname})
		}
	})
	//网站地图相关
	r.GET("/robots.txt", func(c *gin.Context) {
		host := c.Request.Host
		//获取网站id
		sj, _ := webconfig.R(host)
		id := sj["id"]
		dir := sj["dir"]
		muban := sj["muban"]
		if id != "" && dir != "" && muban != "" {
			sj := get_info(id) //网站数据
			if sj == nil {
				c.HTML(http.StatusNotFound, "404.htm", gin.H{"title": "404 - " + webname})
			} else {
				c.File("./map/" + dir + "/robots.txt")
			}
		} else {
			c.HTML(http.StatusNotFound, "404.htm", gin.H{"title": "404 - " + webname})
		}
	})
	r.GET("/sitemap.txt", func(c *gin.Context) {
		host := c.Request.Host
		//获取网站id
		sj, _ := webconfig.R(host)
		id := sj["id"]
		dir := sj["dir"]
		muban := sj["muban"]
		if id != "" && dir != "" && muban != "" {
			sj := get_info(id) //网站数据
			if sj == nil {
				c.HTML(http.StatusNotFound, "404.htm", gin.H{"title": "404 - " + webname})
			} else {
				c.File("./map/" + dir + "/sitemap.txt")
			}
		} else {
			c.HTML(http.StatusNotFound, "404.htm", gin.H{"title": "404 - " + webname})
		}
	})
	r.GET("/sitemap.xml", func(c *gin.Context) {
		host := c.Request.Host
		//获取网站id
		sj, _ := webconfig.R(host)
		id := sj["id"]
		dir := sj["dir"]
		muban := sj["muban"]
		if id != "" && dir != "" && muban != "" {
			sj := get_info(id) //网站数据
			if sj == nil {
				c.HTML(http.StatusNotFound, "404.htm", gin.H{"title": "404 - " + webname})
			} else {
				c.File("./map/" + dir + "/sitemap.xml")
			}
		} else {
			c.HTML(http.StatusNotFound, "404.htm", gin.H{"title": "404 - " + webname})
		}
	})
	r.GET("/sitemap.html", func(c *gin.Context) {
		host := c.Request.Host
		//获取网站id
		sj, _ := webconfig.R(host)
		id := sj["id"]
		dir := sj["dir"]
		muban := sj["muban"]
		if id != "" && dir != "" && muban != "" {
			sj := get_info(id) //网站数据
			if sj == nil {
				c.HTML(http.StatusNotFound, "404.htm", gin.H{"title": "404 - " + webname})
			} else {
				c.File("./map/" + dir + "/sitemap.html")
			}
		} else {
			c.HTML(http.StatusNotFound, "404.htm", gin.H{"title": "404 - " + webname})
		}
	})
	//栏目
	r.GET("/c/:rest", func(c *gin.Context) {
		host := c.Request.Host
		rest := removeNonNumericAnd_(c.Param("rest"))
		//获取网站id
		sj, _ := webconfig.R(host)
		id := sj["id"]
		dir := sj["dir"]
		muban := sj["muban"]
		if id != "" && dir != "" && muban != "" && rest != "" {
			sj := get_list(id, rest) //栏目数据
			if sj == nil {
				c.HTML(http.StatusNotFound, "404.htm", gin.H{"title": "404 - " + webname})
			} else {
				var mubanname string
				if sj["c_lx"] == "1" {
					mubanname = sj["c_mubanl"]
				} else {
					mubanname = sj["c_mubani"]
				}
				//判断是否开启移动端
				if sj["w_wap"] == "1" {
					userAgent := c.Request.Header.Get("User-Agent")
					if strings.Contains(userAgent, "Mobile") || strings.Contains(userAgent, "Android") || strings.Contains(userAgent, "iPhone") {
						mubanname = strings.Replace(mubanname, ".htm", "_m.htm", 1)
					}
				}
				c.HTML(http.StatusOK, dir+"/"+muban+"/"+mubanname, sj)
			}
		} else {
			c.HTML(http.StatusOK, "404.htm", gin.H{"title": "404 - " + webname})
		}
	})
	//文章
	r.GET("/a/:rest", func(c *gin.Context) {
		host := c.Request.Host
		rest := removeNonNumeric(c.Param("rest"))
		//获取网站id
		sj, _ := webconfig.R(host)
		id := sj["id"]
		dir := sj["dir"]
		muban := sj["muban"]
		if id != "" && dir != "" && muban != "" && rest != "" {
			sj := get_art(id, rest) //文章数据
			if sj == nil {
				c.HTML(http.StatusNotFound, "404.htm", gin.H{"title": "404 - " + webname})
			} else {
				var mubanname string = sj["c_mubana"]
				//判断是否开启移动端
				if sj["w_wap"] == "1" {
					userAgent := c.Request.Header.Get("User-Agent")
					if strings.Contains(userAgent, "Mobile") || strings.Contains(userAgent, "Android") || strings.Contains(userAgent, "iPhone") {
						mubanname = strings.Replace(mubanname, ".htm", "_m.htm", 1)
					}
				}
				c.HTML(http.StatusOK, dir+"/"+muban+"/"+mubanname, sj)
			}
		} else {
			c.HTML(http.StatusNotFound, "404.htm", gin.H{"title": "404 - " + webname})
		}
	})
	//搜索
	r.GET("/s/:rest", func(c *gin.Context) {
		host := c.Request.Host
		rest := removeNonChinese(c.Param("rest"))
		//获取网站id
		sj, _ := webconfig.R(host)
		id := sj["id"]
		dir := sj["dir"]
		muban := sj["muban"]
		qp := strings.Split(rest, "_") //转切片
		yema := 1
		if len(qp) > 1 && qp[1] != "" {
			yema2, err := strconv.Atoi(qp[1])
			if err == nil {
				yema = yema2
				if yema < 1 {
					yema = 1
				}
			}
		}
		if id != "" && dir != "" && muban != "" && qp[0] != "" {
			sj := get_info(id) //网站数据
			if sj == nil {
				c.HTML(http.StatusNotFound, "404.htm", gin.H{"title": "404 - " + webname})
			} else {
				sj["s_key"] = qp[0]
				sj["s_yema"] = strconv.Itoa(yema)
				sj["s_yema2"] = ""
				sj["c_id"] = ""
				if yema > 1 {
					sj["s_yema2"] = ` 第` + strconv.Itoa(yema) + `页`
				}
				var mubanname string = "search.htm"
				//判断是否开启移动端
				if sj["w_wap"] == "1" {
					userAgent := c.Request.Header.Get("User-Agent")
					if strings.Contains(userAgent, "Mobile") || strings.Contains(userAgent, "Android") || strings.Contains(userAgent, "iPhone") {
						mubanname = strings.Replace(mubanname, ".htm", "_m.htm", 1)
					}
				}
				c.HTML(http.StatusOK, dir+"/"+muban+"/"+mubanname, sj)
			}
		} else {
			c.HTML(http.StatusNotFound, "404.htm", gin.H{"title": "404 - " + webname})
		}
	})
	//表单
	r.POST("/p", func(c *gin.Context) {
		host := c.Request.Host
		//获取网站id
		sj, _ := webconfig.R(host)
		id := sj["id"]
		dir := sj["dir"]
		muban := sj["muban"]
		if id != "" && dir != "" && muban != "" {
			postsj := map[string]string{}
			ok := true
			for _, v := range postzd {
				if v == "time" {
					postsj[v] = time.Now().Format("2006-01-02 15:04:05")
					continue
				}
				lssj := html.EscapeString(c.PostForm(v))
				if lssj == "" {
					ok = false
					break
				}
				if len(lssj) > 300 {
					lssj = lssj[:300]
				}
				postsj[v] = lssj
			}
			if ok {
				clientIP := c.Request.Header.Get("X-Real-IP") // 如果 X-Real-IP 头为空，获取 X-Forwarded-For
				if clientIP == "" {
					clientIP = c.Request.Header.Get("X-Forwarded-For") // 如果 X-Forwarded-For 头为空，获取 RemoteAddr
					if clientIP == "" {
						clientIP = c.Request.RemoteAddr // RemoteAddr
					}
				}
				//获取时间戳
				sjc := time.Now().Unix()
				sj, _ := ipjilu.R(clientIP)
				if sj == 0 || sj < sjc-60 {
					ipjilu.W(clientIP, sjc)
					laweb := web.R(id)
					if laweb != nil {
						b := laweb["post"]
						var sj []map[string]string
						json.Unmarshal([]byte(b), &sj)
						sj = append([]map[string]string{postsj}, sj...)
						b2, _ := json.Marshal(sj)
						laweb["post"] = string(b2)
						web.W(id, laweb)
						c.Data(http.StatusOK, "text/html; charset=utf-8", []byte(`<title>表单提交...</title><meta name="viewport" content="width=device-width,minimum-scale=1,initial-scale=1"><center><br><br>提交成功！</center><center><br><a href="/">返回首页</a><center><br><a href="javascript:history.back()">返回上页</a></center>`)) //直接返回html
					} else {
						c.Data(http.StatusOK, "text/html; charset=utf-8", []byte(`<title>表单提交...</title><meta name="viewport" content="width=device-width,minimum-scale=1,initial-scale=1"><center><br><br>提交失败！</center><center><br><a href="/">返回首页</a><center><br><a href="javascript:history.back()">返回上页</a></center>`))
					}
				} else {
					t := 60 - (sjc - sj)
					c.Data(http.StatusOK, "text/html; charset=utf-8", []byte(`<title>表单提交...</title><meta name="viewport" content="width=device-width,minimum-scale=1,initial-scale=1"><center><br><br>提交频繁，请`+strconv.Itoa(int(t))+`秒后再试！</center><center><br><a href="/">返回首页</a><center><br><a href="javascript:history.back()">返回上页</a></center>`))
				}
			} else {
				c.Data(http.StatusOK, "text/html; charset=utf-8", []byte(`<title>表单提交...</title><meta name="viewport" content="width=device-width,minimum-scale=1,initial-scale=1"><center><br><br>提交失败！</center><center><br><a href="/">返回首页</a><center><br><a href="javascript:history.back()">返回上页</a></center>`))
			}

		} else {
			c.HTML(http.StatusNotFound, "404.htm", gin.H{"title": "404 - " + webname})
		}
	})
	//后台入口
	r.GET("/"+safeurl, func(c *gin.Context) {
		host := c.Request.Host
		fmt.Println(host)
		c.HTML(http.StatusOK, "houtai.htm", gin.H{
			"safeurl": safeurl,
		})
	})
	r.GET("/ws"+safeurl, func(c *gin.Context) {
		wsHandler(c) //websocket 连接后台交互
	})
	// 处理未找到的路由
	r.NoRoute(func(c *gin.Context) {
		c.HTML(http.StatusNotFound, "404.htm", gin.H{
			"title": "404 - " + webname,
		})
	})
	r.Run("0.0.0.0:" + port) //端口访问
}

func wsHandler(c *gin.Context) {
	// 升级 HTTP 连接为 WebSocket 连接
	con, err := upgrader.Upgrade(c.Writer, c.Request, nil)
	if err != nil {
		log.Println("Failed to set websocket upgrade:", err)
		return
	}
	// 创建连接实例并设置 UID sid
	conn := &Conn{
		Conn: con,
		Uid:  "",
		Wid:  "",
	}
	defer conn.Close()
	conn.SetReadLimit(1024 * 1024 * 100) // 设置消息读取限制 100MB
	go readMessages(conn)                // 启动 Goroutine 处理消息接收
	// 等待连接关闭
	select {}
}

// 关闭 WebSocket 连接
func closeConnection(conn *Conn) {
	if conn.Uid != "" {
		user.D(conn.Uid) //删除bigcache中当前连接用户
	}
	err := conn.Close()
	if err != nil {
		log.Println("关闭连接出错:", err)
	} else {
		log.Printf("连接已关闭")
	}
}

// 更新模板
func updateTemplate() string {
	var result string
	defer func() {
		if rs := recover(); rs != nil {

		}
	}()
	var t *template.Template
	var err error
	t, err = template.New("").Funcs(template.FuncMap{
		"getnavigation":   getnavigation,
		"getarticle":      getarticle,
		"getflink":        getflink,
		"tohtml":          tohtml,
		"totime":          totime,
		"getchanneltitle": getchanneltitle,
		"getgxlanmu":      getgxlanmu,
		"getsearchlist":   getsearchlist,
	}).ParseGlob("templates/**/**/*.htm")
	if err != nil {
		result = fmt.Sprintf("模板解析失败: %v", err)
		log.Println(result)
		return result
	}
	r.SetHTMLTemplate(t)
	result = "1"
	return result
}
